@@ -1,4 +1,4 @@
-/* -*- mode: C -*- Time-stamp: "2011-02-08 01:02:01 jemarch"
+/* -*- mode: C -*-
  *
  *       File:         rec-int.c
  *       Date:         Thu Jul 15 18:23:26 2010
@@ -7,7 +7,7 @@
  *
  */
 
-/* Copyright (C) 2010 Jose E. Marchesi */
+/* Copyright (C) 2010, 2011 Jose E. Marchesi */
 
 /* This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -58,6 +58,12 @@ static int rec_int_check_record_unique (rec_rset_t rset, rec_record_t record,
                                         rec_buf_t errors);
 static int rec_int_check_record_prohibit (rec_rset_t rset, rec_record_t record,
                                           rec_buf_t errors);
+
+#if defined REC_CRYPT_SUPPORT
+static int rec_int_check_record_secrets (rec_rset_t rset, rec_record_t record,
+                                         rec_buf_t errors);
+#endif
+
 static int rec_int_merge_remote (rec_rset_t rset, rec_buf_t errors);
 static bool rec_int_rec_type_p (char *str);
 
@@ -216,8 +222,12 @@ rec_int_check_record (rec_db_t db,
     + rec_int_check_record_types     (db, rset, record, errors)
     + rec_int_check_record_mandatory (rset, record, errors)
     + rec_int_check_record_unique    (rset, record, errors)
+#if defined REC_CRYPT_SUPPORT
+    + rec_int_check_record_secrets   (rset, record, errors)
+#endif
     + rec_int_check_record_prohibit  (rset, record, errors);
 
+
   return res;
 }
 
@@ -228,7 +238,6 @@ rec_int_check_field_type (rec_db_t db,
                           rec_buf_t errors)
 {
   bool res;
-  rec_type_reg_t type_reg;
   rec_field_name_t field_name;
   const char *rset_name;
   rec_rset_t referred_rset;
@@ -274,20 +283,13 @@ rec_int_check_field_type (rec_db_t db,
 
       if (referred_rset)
         {
-          type_reg = rec_rset_get_type_reg (referred_rset);
-          if (type_reg)
-            {
-              referred_type = rec_type_reg_get (type_reg, rec_field_name (field));
-            }
+          referred_type = rec_rset_get_field_type (referred_rset,
+                                                   rec_field_name (field));
         }
     }
 
   /* Get the referring type, if any.  */
-  type_reg = rec_rset_get_type_reg (rset);
-  if (type_reg)
-    {
-      referring_type = rec_type_reg_get (type_reg, rec_field_name (field));
-    }
+  referring_type = rec_rset_get_field_type (rset, rec_field_name (field));
 
   /* The referring type takes precedence.  */
   if (referring_type)
@@ -546,6 +548,49 @@ rec_int_check_record_prohibit (rec_rset_t rset,
   return res;
 }
 
+#if defined REC_CRYPT_SUPPORT
+
+static int
+rec_int_check_record_secrets (rec_rset_t rset,
+                              rec_record_t record,
+                              rec_buf_t errors)
+{
+  int res;
+  rec_field_t field;
+  rec_record_elem_t rec_elem;
+  char *tmp;
+
+  res = 0;
+
+  rec_elem = rec_record_null_elem ();
+  while (rec_record_elem_p (rec_elem = rec_record_next_field (record, rec_elem)))
+    {
+      field = rec_record_elem_field (rec_elem);
+
+      /* If the field is confidential it must be encrypted.  Encrypted
+         field values can be recognized by the "encrypted-"
+         prefix.  */
+#define REC_ENCRYPTED_PREFIX "encrypted-"
+      if (rec_rset_field_confidential_p (rset, rec_field_name (field))
+          && (strncmp (rec_field_value (field),
+                       REC_ENCRYPTED_PREFIX,
+                       strlen (REC_ENCRYPTED_PREFIX)) != 0))
+        {
+          asprintf (&tmp,
+                    _("%s:%s: error: confidential field is not encrypted\n"),
+                    rec_record_source (record),
+                    rec_record_location_str (record));
+          rec_buf_puts (tmp, errors);
+          free (tmp);
+          res++;
+        }
+    }
+
+  return res;
+}
+
+#endif /* REC_CRYPT_SUPPORT */
+
 static int
 rec_int_check_record_key (rec_rset_t rset,
                           rec_record_t orig_record,
@@ -677,19 +722,25 @@ rec_int_check_descriptor (rec_rset_t rset,
   rec_field_name_t rec_fname;
   rec_field_name_t key_fname;
   rec_field_name_t type_fname;
+  rec_field_name_t typedef_fname;
   rec_field_name_t mandatory_fname;
   rec_field_name_t unique_fname;
   rec_field_name_t prohibit_fname;
   rec_field_name_t auto_fname;
   rec_field_name_t size_fname;
+  rec_field_name_t sort_fname;
+#if defined REC_CRYPT_SUPPORT
+  rec_field_name_t confidential_fname;
+#endif
   char *field_value;
   rec_fex_t fex;
   char *tmp;
   rec_field_name_t auto_field_name;
   char *auto_field_name_str;
   size_t i;
-  rec_type_reg_t type_reg;
   rec_type_t type;
+  char *type_name = NULL;
+  char *p, *q = NULL;
 
   res = 0;
 
@@ -700,11 +751,16 @@ rec_int_check_descriptor (rec_rset_t rset,
       rec_fname = rec_parse_field_name_str ("%rec:");
       key_fname = rec_parse_field_name_str ("%key:");
       type_fname = rec_parse_field_name_str ("%type:");
+      typedef_fname = rec_parse_field_name_str ("%typedef:");
       mandatory_fname = rec_parse_field_name_str ("%mandatory:");
       unique_fname = rec_parse_field_name_str ("%unique:");
       prohibit_fname = rec_parse_field_name_str ("%prohibit:");
       auto_fname = rec_parse_field_name_str ("%auto:");
       size_fname = rec_parse_field_name_str ("%size:");
+      sort_fname = rec_parse_field_name_str ("%sort:");
+#if defined REC_CRYPT_SUPPORT
+      confidential_fname = rec_parse_field_name_str ("%confidential:");
+#endif
 
       /* Check the type of the record set:
 
@@ -770,6 +826,18 @@ rec_int_check_descriptor (rec_rset_t rset,
           res++;
         }
 
+      /* Only one 'sort:' entry is allowed, if any.  */
+      if (rec_record_get_num_fields_by_name (descriptor, sort_fname) > 1)
+        {
+          asprintf (&tmp,
+                    _("%s:%s: error: only one %%sort field is allowed in a record descriptor\n"),
+                    rec_record_source (descriptor),
+                    rec_record_location_str (descriptor));
+          rec_buf_puts (tmp, errors);
+          free (tmp);
+          res++;
+        }
+
       /* Iterate on fields.  */
       rec_elem = rec_record_null_elem ();
       while (rec_record_elem_p (rec_elem = rec_record_next_field (descriptor, rec_elem)))
@@ -781,18 +849,122 @@ rec_int_check_descriptor (rec_rset_t rset,
 
           if (rec_field_name_equal_p (field_name, type_fname))
             {
-              /* Check the type descriptor.  */
-              if (!rec_type_descr_p (field_value))
+              /* Check for the list of fields.  */
+              p = field_value;
+              rec_skip_blanks (&p);
+              if (!rec_parse_regexp (&p, "^" REC_FNAME_RE "(," REC_FNAME_RE ")*",
+                                     NULL))
                 {
-                  /* XXX: make rec_type_descr_p to report more details.  */
                   asprintf (&tmp,
-                             _("%s:%s: error: invalid type specification\n"),
-                             rec_field_source (field),
-                             rec_field_location_str (field));
+                            _("%s:%s: error: expected a comma-separated list of fields \
+before the type specification\n"),
+                            rec_field_source (field),
+                            rec_field_location_str (field));
                   rec_buf_puts (tmp, errors);
                   free (tmp);
                   res++;
                 }
+
+              /* Check the type descriptor.  Note that it can be
+                 either a type specification or a type name.  */
+              rec_skip_blanks (&p);
+              if (!rec_type_descr_p (p))
+                {
+                  q = p;
+                  if (rec_parse_regexp (&q, "^" REC_TYPE_NAME_RE "[ \t\n]*$",
+                                        NULL))
+                    {
+                      /* The named type shall exist in the record set
+                         type registry.
+                      
+                         XXX: but this is probably a warning rather
+                         than an error.  */
+
+                      rec_parse_regexp (&p, "^" REC_TYPE_NAME_RE, &type_name);
+                      if (!rec_type_reg_get (rec_rset_get_type_reg (rset), type_name))
+                        {
+                          asprintf (&tmp,
+                                    _("%s:%s: error: the referred type %s \
+does not exist\n"),
+                                    rec_field_source (field),
+                                    rec_field_location_str (field),
+                                    type_name);
+                          rec_buf_puts (tmp, errors);
+                          free (tmp);
+                          res++;
+                        }
+                    }
+                  else
+                    {
+                      /* XXX: make rec_type_descr_p to report more details.  */
+                      asprintf (&tmp,
+                                _("%s:%s: error: invalid typedef specification\n"),
+                                rec_field_source (field),
+                                rec_field_location_str (field));
+                      rec_buf_puts (tmp, errors);
+                      free (tmp);
+                      res++;
+                    }
+                }
+            }
+          else if (rec_field_name_equal_p (field_name, typedef_fname))
+            {
+              /* Check for the type name.  */
+              p = field_value;
+              rec_skip_blanks (&p);
+              if (!rec_parse_regexp (&p, "^" REC_TYPE_NAME_RE, NULL))
+                {
+                  asprintf (&tmp,
+                            _("%s:%s: error: expected a type name before the type \
+specification\n"),
+                            rec_field_source (field),
+                            rec_field_location_str (field));
+                  rec_buf_puts (tmp, errors);
+                  free (tmp);
+                  res++;
+                }
+              
+              /* Check the type descriptor.  Note that it can be
+                 either a type specification or a type name.  */
+              rec_skip_blanks (&p);
+              if (!rec_type_descr_p (p))
+                {
+                  q = p;
+                  if (rec_parse_regexp (&q, "^" REC_TYPE_NAME_RE "[ \t\n]*$",
+                                        NULL))
+                    {
+                      /* The named type shall exist in the record set
+                         type registry.
+                      
+                         XXX: but this is probably a warning rather
+                         than an error.  */
+
+                      rec_parse_regexp (&p, "^" REC_TYPE_NAME_RE, &type_name);
+                      if (!rec_type_reg_get (rec_rset_get_type_reg (rset), type_name))
+                        {
+                          asprintf (&tmp,
+                                    _("%s:%s: error: the referred type %s \
+does not exist\n"),
+                                    rec_field_source (field),
+                                    rec_field_location_str (field),
+                                    type_name);
+                          rec_buf_puts (tmp, errors);
+                          free (tmp);
+                          res++;
+                        }
+                    }
+                  else
+                    {
+                      /* XXX: make rec_type_descr_p to report more details.  */
+                      asprintf (&tmp,
+                                _("%s:%s: error: invalid typedef specification\n"),
+                                rec_field_source (field),
+                                rec_field_location_str (field));
+                      rec_buf_puts (tmp, errors);
+                      free (tmp);
+                      res++;
+                    }
+                }
             }
           else if (rec_field_name_equal_p (field_name, mandatory_fname)
                    || rec_field_name_equal_p (field_name, unique_fname)
@@ -819,6 +991,23 @@ rec_int_check_descriptor (rec_rset_t rset,
                   res++;
                 }
             }
+          else if (rec_field_name_equal_p (field_name, sort_fname))
+            {
+              if (!rec_match (field_value,
+                              "^"
+                              "[ \n\t]*" REC_FNAME_RE "[ \n\t]*"
+                              "$"))
+                {
+                  asprintf (&tmp,
+                            _("%s:%s: error: value for %s shall be a field name.\n"),
+                            rec_field_source (field),
+                            rec_field_location_str (field),
+                            field_name_str);
+                  rec_buf_puts (tmp, errors);
+                  free (tmp);
+                  res++ ;
+                }
+            }
           else if (rec_field_name_equal_p (field_name, size_fname))
             {
               if (!rec_match (field_value, REC_INT_SIZE_RE))
@@ -833,8 +1022,26 @@ rec_int_check_descriptor (rec_rset_t rset,
                   res++;
                 }
             }
+#if defined REC_CRYPT_SUPPORT
+          else if (rec_field_name_equal_p (field_name, confidential_fname))
+            {
+              if (!rec_match (field_value,
+                              "^"
+                              "[ \n\t]*" REC_FNAME_RE "([ \n\t]+" REC_FNAME_RE ")*"
+                              "[ \n\t]*$"))
+                {
+                  asprintf (&tmp,
+                            _("%s:%s: error: value for %s shall be a list of field names.\n"),
+                            rec_field_source (field),
+                            rec_field_location_str (field),
+                            field_name_str);
+                  rec_buf_puts (tmp, errors);
+                  free (tmp);
+                  res++;
+                }
+            }
+#endif /* REC_CRYPT_SUPPORT */          
 
-          
           if ((rec_field_name_equal_p (field_name, auto_fname))
               && (fex = rec_fex_new (field_value, REC_FEX_SIMPLE)))
             {
@@ -844,24 +1051,20 @@ rec_int_check_descriptor (rec_rset_t rset,
                 {
                   auto_field_name = rec_fex_elem_field_name (rec_fex_get (fex, i));
                   auto_field_name_str = rec_fex_elem_field_name_str (rec_fex_get (fex, i));
-                  type_reg = rec_rset_get_type_reg (rset);
-                  if (type_reg)
+                  type = rec_rset_get_field_type (rset, auto_field_name);
+                  if ((!type) ||
+                      ! ((rec_type_kind (type) == REC_TYPE_INT)
+                         || (rec_type_kind (type) == REC_TYPE_RANGE)
+                         || (rec_type_kind (type) == REC_TYPE_DATE)))
                     {
-                      type = rec_type_reg_get (type_reg, auto_field_name);
-                      if ((!type) ||
-                          ! ((rec_type_kind (type) == REC_TYPE_INT)
-                             || (rec_type_kind (type) == REC_TYPE_RANGE)
-                             || (rec_type_kind (type) == REC_TYPE_DATE)))
-                        {
-                          asprintf (&tmp,
-                                    _("%s:%s: error: auto-incremented field %s shall be of type int, range or date\n"),
-                                    rec_record_source (descriptor),
-                                    rec_record_location_str (descriptor),
-                                    auto_field_name_str);
-                          rec_buf_puts (tmp, errors);
-                          free (tmp);
-                          res++;
-                        }
+                      asprintf (&tmp,
+                                _("%s:%s: error: auto-incremented field %s shall be of type int, range or date\n"),
+                                rec_record_source (descriptor),
+                                rec_record_location_str (descriptor),
+                                auto_field_name_str);
+                      rec_buf_puts (tmp, errors);
+                      free (tmp);
+                      res++;
                     }
                 }
             }
@@ -871,11 +1074,16 @@ rec_int_check_descriptor (rec_rset_t rset,
       rec_field_name_destroy (rec_fname);
       rec_field_name_destroy (key_fname);
       rec_field_name_destroy (type_fname);
+      rec_field_name_destroy (typedef_fname);
       rec_field_name_destroy (mandatory_fname);
       rec_field_name_destroy (unique_fname);
       rec_field_name_destroy (prohibit_fname);
       rec_field_name_destroy (auto_fname);
       rec_field_name_destroy (size_fname);
+      rec_field_name_destroy (sort_fname);
+#if defined REC_CRYPT_SUPPORT
+      rec_field_name_destroy (confidential_fname);
+#endif
     }
 
   return res;
