@@ -99,7 +99,7 @@
   "$"
   
 /* Regular expression denoting a type name.  */
-#define REC_TYPE_NAME_RE                               \
+#define REC_TYPE_CLASS_RE                               \
   "(" REC_TYPE_INT_NAME   "|" REC_TYPE_RANGE_NAME  "|" \
       REC_TYPE_REAL_NAME  "|" REC_TYPE_SIZE_NAME   "|" \
       REC_TYPE_LINE_NAME  "|" REC_TYPE_REGEXP_NAME "|" \
@@ -195,6 +195,8 @@
 
 struct rec_type_s
 {
+  char *name;                 /* Name of the type.  May be NULL in an
+                                 anonymous type.  */
   enum rec_type_kind_e kind;  /* Kind of the type.  */
   char *expr;                 /* Copy of the type descriptor used to
                                  create the type.  */
@@ -209,18 +211,21 @@ struct rec_type_s
   } data;
 };
 
+#define REC_TYPE_REG_MAX_TYPES  100
+
 struct rec_type_reg_entry_s
 {
-  rec_field_name_t name;
-  rec_type_t type;
-};
+  char *type_name;
 
-#define REC_TYPE_REG_MAX_ENTRIES 100
+  rec_type_t type;
+  char *to_type;
+  bool visited_p;
+};
 
 struct rec_type_reg_s
 {
-  size_t num_entries;
-  struct rec_type_reg_entry_s entries[REC_TYPE_REG_MAX_ENTRIES];
+  size_t num_types;
+  struct rec_type_reg_entry_s types[REC_TYPE_REG_MAX_TYPES];
 };
 
 /*
@@ -271,35 +276,6 @@ rec_type_descr_p (char *str)
   return ret;
 }
 
-rec_fex_t
-rec_type_descr_fex (char *str)
-{
-  rec_fex_t fex = NULL;
-  char *p;
-  char *name;
-
-  if (!rec_type_descr_p (str))
-    {
-      return NULL;
-    }
-
-  p = str;
-
-  /* Skip blank characters.  */
-  rec_skip_blanks (&p);
-
-  /* Get the FEX.  */
-  if (rec_parse_regexp (&p,
-                        "^" REC_FNAME_RE "(," REC_FNAME_RE ")*",
-                        &name))
-    {
-      fex = rec_fex_new (name, REC_FEX_CSV); 
-      free (name);
-    }
-
-  return fex;
-}
-
 char *
 rec_type_descr_type (char *str)
 {
@@ -336,8 +312,7 @@ rec_type_new (char *str)
 {
   rec_type_t new;
   char *p;
-  char *field_name_str = NULL;
-  char *type_name_str = NULL;
+  char *type_kind_str = NULL;
 
   p = str;
   new = malloc (sizeof (struct rec_type_s));
@@ -345,21 +320,12 @@ rec_type_new (char *str)
     {
       goto exit;
     }
+  new->name = NULL; /* Newly created types are anonyous. */
 
-  /* Skip the field name surrounded by blanks.  */
-  rec_skip_blanks (&p);
-  if (!rec_parse_regexp (&p,
-                              "^" REC_FNAME_RE "(," REC_FNAME_RE ")*",
-                              &field_name_str))
-    {
-      free (new);
-      new = NULL;
-      goto exit;
-    }
   rec_skip_blanks (&p);
 
-  /* Get the type name.  */
-  if (!rec_parse_regexp (&p, "^" REC_TYPE_NAME_RE, &type_name_str))
+  /* Get the type kind.  */
+  if (!rec_parse_regexp (&p, "^" REC_TYPE_CLASS_RE, &type_kind_str))
     {
       free (new);
       new = NULL;
@@ -367,7 +333,7 @@ rec_type_new (char *str)
     }
 
   /* Continue parsing depending on the kind of type.  */
-  new->kind = rec_type_parse_type_kind (type_name_str);
+  new->kind = rec_type_parse_type_kind (type_kind_str);
   switch (new->kind)
     {
     case REC_TYPE_SIZE:
@@ -450,8 +416,7 @@ rec_type_new (char *str)
 
  exit:
 
-  free (field_name_str);
-  free (type_name_str);
+  free (type_kind_str);
 
   return new;
 }
@@ -635,6 +600,7 @@ rec_type_check (rec_type_t type,
 void
 rec_type_destroy (rec_type_t type)
 {
+  free (type->name);
   free (type);
 }
 
@@ -646,7 +612,7 @@ rec_type_reg_new (void)
   new = malloc (sizeof (struct rec_type_reg_s));
   if (new)
     {
-      new->num_entries = 0;
+      new->num_types = 0;
     }
 
   return new;
@@ -656,61 +622,143 @@ void
 rec_type_reg_destroy (rec_type_reg_t reg)
 {
   size_t i;
-
-  for (i = 0; i < reg->num_entries; i++)
-    {
-      rec_field_name_destroy (reg->entries[i].name);
-      rec_type_destroy (reg->entries[i].type);
-    }
   
+  for (i = 0; i < reg->num_types; i++)
+    {
+      if (reg->types[i].type)
+        {
+          rec_type_destroy (reg->types[i].type);
+        }
+      free (reg->types[i].type_name);
+      free (reg->types[i].to_type);
+    }
   free (reg);
 }
 
 void
-rec_type_reg_register (rec_type_reg_t reg,
-                       rec_field_name_t name,
-                       rec_type_t type)
+rec_type_reg_add (rec_type_reg_t reg,
+                  rec_type_t type)
 {
   size_t i;
+  const char *type_name = NULL;
 
-  for (i = 0; i < reg->num_entries; i++)
+  type_name = rec_type_name (type);
+  if (!type_name)
     {
-      if (rec_field_name_equal_p (reg->entries[i].name, name)
-          || rec_field_name_ref_p (reg->entries[i].name, name))
+      /* The registry only contains named types.  */
+      return;
+    }
+
+  for (i = 0; i < reg->num_types; i++)
+    {
+      if (strcmp (reg->types[i].type_name, type_name) == 0)
         {
           /* Replace this entry.  */
+          if (reg->types[i].type)
+            {
+              rec_type_destroy (reg->types[i].type);
+            }
+          free (reg->types[i].type_name);
+          free (reg->types[i].to_type);
           break;
         }
     }
 
-  reg->entries[i].name = rec_field_name_dup (name);
-  reg->entries[i].type = type;
-  if (i == reg->num_entries)
+  reg->types[i].type_name = strdup (rec_type_name (type));
+  reg->types[i].type = type;
+  reg->types[i].to_type = NULL;
+  reg->types[i].visited_p = false;
+  if (i == reg->num_types)
     {
       /* We added a new entry.  */
-      reg->num_entries++;
+      reg->num_types++;
+    }
+}
+
+void
+rec_type_reg_add_synonym (rec_type_reg_t reg,
+                          const char *type_name,
+                          const char *to_type)
+{
+  size_t i;
+
+  for (i = 0; i < reg->num_types; i++)
+    {
+      if (strcmp (reg->types[i].type_name, type_name) == 0)
+        {
+          /* Replace this entry.  */
+          if (reg->types[i].type)
+            {
+              rec_type_destroy (reg->types[i].type);
+            }
+          free (reg->types[i].type_name);
+          free (reg->types[i].to_type);
+          break;
+        }
+    }
+
+  reg->types[i].type_name = strdup (type_name);
+  reg->types[i].to_type = strdup (to_type);
+  reg->types[i].type = NULL;
+  reg->types[i].visited_p = false;
+  if (i == reg->num_types)
+    {
+      /* We added a new entry.  */
+      reg->num_types++;
     }
 }
 
 rec_type_t
 rec_type_reg_get (rec_type_reg_t reg,
-                  rec_field_name_t name)
+                  const char *type_name)
 {
-  rec_type_t res;
   size_t i;
+  rec_type_t type = NULL;
 
-  res = NULL;
-  for (i = 0; i < reg->num_entries; i++)
+  for (i = 0; i < reg->num_types; i++)
     {
-      if (rec_field_name_equal_p (reg->entries[i].name, name)
-          || rec_field_name_ref_p (reg->entries[i].name, name))
+      if (strcmp (reg->types[i].type_name, type_name) == 0)
         {
-          res = reg->entries[i].type;
-          break;
+          if (reg->types[i].type)
+            {
+              /* Type found.  */
+              type = reg->types[i].type;
+              break;
+            }
+          else
+            {
+              /* Loop detection.  */
+              if (reg->types[i].visited_p)
+                {
+                  break;
+                }
+              
+              /* Mark this entry as visited and follow the name.  */
+              reg->types[i].visited_p = true;
+              type = rec_type_reg_get (reg, reg->types[i].to_type);
+            }
         }
     }
 
-  return res;
+  /* Reset the visited flags.  */
+  for (i = 0; i < reg->num_types; i++)
+    {
+      reg->types[i].visited_p = false;
+    }
+  
+  return type;
+}
+
+const char *
+rec_type_name (rec_type_t type)
+{
+  return type->name;
+}
+
+void
+rec_type_set_name (rec_type_t type, const char *name)
+{
+  type->name = strdup (name);
 }
 
 bool
