@@ -209,12 +209,15 @@ rec_parse_regexp (char **str, char *re, char **result)
       /* Try to match the regexp.  */
       if (regexec (&regexp, p, 1, &pm, 0) == 0)
         {
-          /* Get the match into 'result'.  Note that
-             since the pattern starts with a ^ rm_so shall be 0 and we
-             can use rm_eo relative to *p.  */
-          *result = malloc (pm.rm_eo + 1);
-          strncpy (*result, p, pm.rm_eo);
-          (*result)[pm.rm_eo] = '\0';
+          if (result)
+            {
+              /* Get the match into 'result'.  Note that
+                 since the pattern starts with a ^ rm_so shall be 0 and we
+                 can use rm_eo relative to *p.  */
+              *result = malloc (pm.rm_eo + 1);
+              strncpy (*result, p, pm.rm_eo);
+              (*result)[pm.rm_eo] = '\0';
+            }
 
           /* Advance 'p'.  */
           p = p + pm.rm_eo;
@@ -290,7 +293,7 @@ rec_extract_size (char *str)
 {
   char *p;
   char *condition_str;
-  size_t res;
+  int res;
 
   if (!rec_match (str, REC_INT_SIZE_RE))
     {
@@ -301,9 +304,9 @@ rec_extract_size (char *str)
   rec_skip_blanks (&p);
   rec_parse_regexp (&p, "^[><]=?", &condition_str);
   rec_skip_blanks (&p);
-  rec_parse_int (&p, (int *) &res);
+  rec_parse_int (&p, &res);
 
-  return res;
+  return (size_t) res;
 }
 
 enum rec_size_condition_e
@@ -356,4 +359,22 @@ rec_extract_size_condition (char *str)
   return condition;
 }
 
+int
+rec_timespec_subtract (struct timespec *result,
+                       struct timespec *x,
+                       struct timespec *y)
+{
+  result->tv_sec = x->tv_sec - y->tv_sec;
+  result->tv_nsec = x->tv_nsec - y->tv_nsec;
+  if (result->tv_nsec < 0)
+    {
+      /* Overflow.  Subtract one second.  */
+      result->tv_sec--;
+      result->tv_nsec += 1000000000;
+    }
+
+  /* Return whether there is an overflow in the 'tv_sec' field.  */
+  return (result->tv_sec < 0);
+}
+
 /* End of rec-utils.c */
