@@ -49,6 +49,9 @@ struct rec_writer_s
 
   bool eof;
   int line;  /* Current line number. */
+
+  char *password;    /* Password to use to decrypt encrypted
+                        fields.  */
 };
 
 rec_writer_t
@@ -63,6 +66,7 @@ rec_writer_new (FILE *file_out)
       new->buf_out = NULL;
       new->line = 1;
       new->eof = false;
+      new->password = NULL;
     }
 
   return new;
@@ -97,6 +101,7 @@ rec_writer_destroy (rec_writer_t writer)
       rec_buf_close (writer->buf_out);
     }
 
+/*  free (writer->password); */
   free (writer);
 }
 
@@ -176,6 +181,18 @@ bool
 rec_write_field (rec_writer_t writer,
                  rec_field_t field,
                  rec_writer_mode_t mode)
+{
+  rec_write_field_with_rset (writer,
+                             NULL,
+                             field,
+                             mode);
+}
+
+bool
+rec_write_field_with_rset (rec_writer_t writer,
+                           rec_rset_t rset,
+                           rec_field_t field,
+                           rec_writer_mode_t mode)
 {
   size_t pos;
   rec_field_name_t fname;
@@ -219,6 +236,13 @@ rec_write_field (rec_writer_t writer,
         }
     }
 
+#if defined REC_CRYPT_SUPPORT
+  if (writer->password)
+    {
+      rec_decrypt_field (field, writer->password);
+    }
+#endif /* REC_CRYPT_SUPPORT */
+
   fvalue = rec_field_value (field);
   for (pos = 0; pos < strlen (fvalue); pos++)
     {
@@ -352,6 +376,18 @@ bool
 rec_write_record (rec_writer_t writer,
                   rec_record_t record,
                   rec_writer_mode_t mode)
+{
+  rec_write_record_with_rset (writer,
+                             NULL,
+                             record,
+                             mode);
+}
+
+bool
+rec_write_record_with_rset (rec_writer_t writer,
+                            rec_rset_t rset,
+                            rec_record_t record,
+                            rec_writer_mode_t mode)
 {
   bool ret;
   rec_field_t field;
@@ -384,7 +420,10 @@ rec_write_record (rec_writer_t writer,
         {
           /* Write a field.  */
           field = rec_record_elem_field (elem);
-          if (!rec_write_field (writer, field, mode))
+          if (!rec_write_field_with_rset (writer,
+                                          rset,
+                                          field,
+                                          mode))
             {
               ret = false;
               break;
@@ -578,7 +617,6 @@ rec_write_rset (rec_writer_t writer,
       position++;
     }
 
-
   /* Special case:
    *
    * # comment 1
@@ -692,6 +730,14 @@ rec_write_comment_str (rec_comment_t comment,
   return result;
 }
 
+void
+rec_writer_set_password (rec_writer_t writer,
+                         char *password)
+{
+  free (writer->password);
+  writer->password = strdup (password);
+}
+
 /*
  * Private functions
  */
