@@ -26,13 +26,11 @@
 #ifndef REC_H
 #define REC_H
 
-#include <config.h>
-
 #include <stdbool.h>
 #include <stdio.h>
 #include <fcntl.h>
 
-/* 
+/*
  * rec format version implemented by this library.
  */
 
@@ -40,7 +38,7 @@
 #define REC_VERSION_MINOR 0
 #define REC_VERSION_STRING "1.0"
 
-/* 
+/*
  * INITIALIZATION of the library
  */
 
@@ -109,6 +107,7 @@ typedef struct rec_field_name_s *rec_field_name_t;
 #define REC_FNAME_RE                                                  \
   REC_FNAME_PART_RE "(" ":" REC_FNAME_PART_RE ")*:?"
 
+#define REC_TYPE_NAME_RE "[a-zA-Z][a-zA-Z0-9_-]*"
 #define REC_URL_REGEXP "(file|http|ftp|https)://[^ \t]+"
 #define REC_FILE_REGEXP "(/?[^/ \t\n]+)+"
 
@@ -138,7 +137,7 @@ rec_field_name_t rec_field_name_dup (rec_field_name_t fname);
  * - Two given field names are EQUAL if and only if their name parts
  *   with the biggest index are identical.  The following names are
  *   thus equal:
- *   
+ *
  *         A:B:C:  .EQUAL. X:Y:C:  .EQUAL. C:
  *
  * - A given field name NAME2 is a reference of a field name NAME1 if
@@ -219,7 +218,8 @@ enum rec_fex_kind_e
 
 /* Create a field expression structure from a string.  A fex kind
    shall be specified in KIND.  If STR does not contain a valid FEX of
-   the given kind then NULL is returned.  */
+   the given kind then NULL is returned.  If STR is NULL then an empty
+   fex is returned.  */
 rec_fex_t rec_fex_new (char *str,
                        enum rec_fex_kind_e kind);
 
@@ -303,15 +303,6 @@ void rec_type_destroy (rec_type_t type);
 /* Determine whether a string contains a valid type description.  */
 bool rec_type_descr_p (char *str);
 
-/* Extract the fex of a string containing a type description.  Return
-   NULL if STR is not a valid type description.  */
-rec_fex_t rec_type_descr_fex (char *str);
-
-/* Extract the type part (everything but the fex) of a string
-   containing a type description.  Return NULL if STR is not a valid
-   type description.  */
-char *rec_type_descr_type (char *str);
-
 /* Get the kind of the type.  The _str version returns a string with
    the name of the type.  */
 enum rec_type_kind_e rec_type_kind (rec_type_t type);
@@ -322,8 +313,14 @@ char *rec_type_kind_str (rec_type_t type);
 int rec_type_min (rec_type_t type);
 int rec_type_max (rec_type_t type);
 
+/* Get and set the name of a type.  Types are created anonymous by
+   rec_type_new, so the getter will return NULL unless a name is
+   set.  */
+const char *rec_type_name (rec_type_t type);
+void rec_type_set_name (rec_type_t type, const char *name);
+
 /* Determine whether two types are the same type.
- * 
+ *
  * Two types are equal if,
  *
  * - They are of the same kind, and
@@ -331,7 +328,7 @@ int rec_type_max (rec_type_t type);
  * - Depending on the kind of types:
  *
  *   + For sizes
- *    
+ *
  *     The maximum size specified in both types is the same.
  *
  *   + For ranges
@@ -356,12 +353,7 @@ bool rec_type_check (rec_type_t type, char *str, char **error_str);
 /*
  * TYPE REGISTRIES.
  *
- * Type registries are collections of associations:
- *
- *     TYPE <-> FIELD NAME
- *
- * Those registries are used in record sets to store the types of some
- * fields, but can be used for other purposes.  The following API
+ * Type registries are collections of named types. The following API
  * provides facilities to maintain type registries.
  */
 
@@ -373,16 +365,19 @@ rec_type_reg_t rec_type_reg_new (void);
 /* Destroy a type registry, freeing resources.  */
 void rec_type_reg_destroy (rec_type_reg_t reg);
 
-/* Add TYPE as the type for the field name NAME in the type registry
-   REG.  If the field NAME is already associated in REG, the previous
-   association gets overwritten.  */
-void rec_type_reg_register (rec_type_reg_t reg, rec_field_name_t name, rec_type_t type);
+/* Insert a new type in the type registry.  If a type with the same
+   name already exists in the registry then it gets replaced.  */
+void rec_type_reg_add (rec_type_reg_t reg, rec_type_t type);
 
-/* Get the type associated with the field NAME.  If no association for
-   that field name exists in the registry then NULL is returned.  Note
-   that the used equality function is: EQUAL || REF.  See above for
-   details on field name equality.  */
-rec_type_t rec_type_reg_get (rec_type_reg_t reg, rec_field_name_t name);
+/* Insert a new type in the type registry as a synonim of another
+   type.  If a type with the same name already exists in the registry
+   then it gets replaced.  */
+void rec_type_reg_add_synonym (rec_type_reg_t reg, const char *type_name,
+                               const char *to_name);
+
+/* Get the type named TYPE_NAME stored in REG.  If it does not exist
+   NULL is returned.  */
+rec_type_t rec_type_reg_get (rec_type_reg_t reg, const char *type_name);
 
 /*
  * FIELDS
@@ -643,7 +638,6 @@ char *rec_rset_url  (rec_rset_t rset);
 void rec_rset_set_type (rec_rset_t rset, char *type);
 
 /* Type registry.  */
-
 rec_type_reg_t rec_rset_get_type_reg (rec_rset_t rset);
 
 /* Types.  */
@@ -662,6 +656,12 @@ void rec_rset_rename_field (rec_rset_t rset,
 /* Get a fex with the auto-incremented fields in this record set.  */
 rec_fex_t rec_rset_auto (rec_rset_t rset);
 
+/* Get a fex with the confidential fields in this record set.  */
+rec_fex_t rec_rset_confidential (rec_rset_t rset);
+
+/* Determine whether fields having a given name are confidential.  */
+bool rec_rset_field_confidential_p (rec_rset_t rset, rec_field_name_t field_name);
+
 /* Get the size constraints of the rset.  */
 size_t rec_rset_min_records (rec_rset_t rset);
 size_t rec_rset_max_records (rec_rset_t rset);
@@ -669,6 +669,14 @@ size_t rec_rset_max_records (rec_rset_t rset);
 /* Locations.  */
 char *rec_rset_source (rec_rset_t rset);
 
+/* Ordered attribute.  */
+void rec_rset_set_ordered (rec_rset_t rset, bool sorted_p);
+bool rec_rset_ordered (rec_rset_t rset);
+
+/* Order_by field.  */
+void rec_rset_set_order_by_field (rec_rset_t rset, rec_field_name_t field_name);
+rec_field_name_t rec_rset_order_by_field (rec_rset_t rset);
+
 /*
  * DATABASES
  *
@@ -806,6 +814,13 @@ bool rec_parser_error (rec_parser_t parser);
 /* Reset the error status and EOF of a parser. */
 void rec_parser_reset (rec_parser_t parser);
 
+/* Parser properties.  */
+void rec_parser_set_ordered (rec_parser_t parser, bool ordered_p);
+bool rec_parser_ordered (rec_parser_t parser);
+void rec_parser_sort_rset (rec_parser_t parser,
+                           char *rset_name,
+                           rec_field_name_t field_name);
+
 /* Print a message with details on the last parser error.
  *
  * This function produces a message on the standard error output,
@@ -828,6 +843,9 @@ typedef struct rec_writer_s *rec_writer_t;
    memory, return NULL. */
 rec_writer_t rec_writer_new (FILE *out);
 
+/* Set the password to use when writing encrypted fields.  */
+void rec_writer_set_password (rec_writer_t writer, char *password);
+
 /* Create a writer associated with a given string.  If not enough
    memory, return NULL.  */
 rec_writer_t rec_writer_new_str (char **str, size_t *str_size);
@@ -848,13 +866,17 @@ enum rec_writer_mode_e
   REC_WRITER_NORMAL,
   REC_WRITER_SEXP
 };
-  
+
 typedef enum rec_writer_mode_e rec_writer_mode_t;
 
 bool rec_write_comment (rec_writer_t writer, rec_comment_t comment, rec_writer_mode_t mode);
 bool rec_write_field_name (rec_writer_t writer, rec_field_name_t field_name, rec_writer_mode_t mode);
 bool rec_write_field (rec_writer_t writer, rec_field_t field, rec_writer_mode_t mode);
+bool rec_write_field_with_rset (rec_writer_t writer, rec_rset_t rset, rec_field_t field,
+                                rec_writer_mode_t mode);
 bool rec_write_record (rec_writer_t writer, rec_record_t record, rec_writer_mode_t mode);
+bool rec_write_record_with_rset (rec_writer_t writer, rec_rset_t rset, rec_record_t record,
+                                 rec_writer_mode_t mode);
 bool rec_write_record_with_fex (rec_writer_t writer, rec_record_t record, rec_fex_t fex,
                                 rec_writer_mode_t mode,
                                 bool print_values_p, bool print_in_a_row_p);
@@ -894,7 +916,43 @@ bool rec_sex_eval (rec_sex_t sex, rec_record_t record, bool *status);
 char *rec_sex_eval_str (rec_sex_t sex, rec_record_t record);
 
 void rec_sex_print_ast (rec_sex_t sex);
-                             
+
+/*
+ * Encryption routines.
+ */
+
+#if defined REC_CRYPT_SUPPORT
+
+#define REC_ENCRYPTED_PREFIX "encrypted-"
+
+bool rec_encrypt (char *in,
+                  size_t in_size,
+                  char *password,
+                  char **out,
+                  size_t *out_size);
+
+bool rec_decrypt (char *in,
+                  size_t in_size,
+                  char *password,
+                  char **out,
+                  size_t *out_size);
+
+bool rec_encrypt_field (rec_field_t field,
+                        char *password);
+
+bool rec_encrypt_record (rec_rset_t rset,
+                         rec_record_t record,
+                         char *password);
+
+bool rec_decrypt_field (rec_field_t field,
+                        char *password);
+
+bool rec_decrypt_record (rec_rset_t rset,
+                         rec_record_t record,
+                         char *password);
+
+#endif /* REC_CRYPT_SUPPORT */
+
 #endif /* !REC_H */
 
 /* End of rec.h */
