@@ -28,7 +28,6 @@
 #include <getopt.h>
 #include <string.h>
 #include <stdlib.h>
-#include <xalloc.h>
 #include <gettext.h>
 #define _(str) gettext (str)
 
@@ -316,6 +315,11 @@ main (int argc, char *argv[])
   recdel_parse_args (argc, argv);
 
   db = recutl_read_db_from_file (recdel_file);
+  if (!db)
+    {
+      recutl_fatal (_("cannot read file %s\n"), recdel_file);
+    }
+
   if (((recutl_num != -1) || recutl_sex) || recdel_force)
     {
       recdel_delete_records (db);
