@@ -1,4 +1,4 @@
-/* -*- mode: C -*- Time-stamp: "2011-02-08 00:56:07 jemarch"
+/* -*- mode: C -*-
  *
  *       File:         recfmt.c
  *       Date:         Wed Dec 22 18:20:20 2010
@@ -111,7 +111,7 @@ recfmt_parse_args (int argc,
                    char **argv)
 {
   char c;
-  char ret;
+  int ret;
 
   while ((ret = getopt_long (argc,
                              argv,
@@ -298,6 +298,7 @@ main (int argc, char *argv[])
   rec_db_t db;
 
   recutl_init ("recfmt");
+  recutl_sorting_parser (true, NULL, NULL);
 
   recfmt_parse_args (argc, argv);
 
