@@ -31,8 +31,10 @@
 #include <xalloc.h>
 #include <gettext.h>
 #include <errno.h>
+#include <locale.h>
 #include <time.h>
 #define _(str) gettext (str)
+#include <base64.h>
 
 #include <rec.h>
 #include <recutl.h>
@@ -56,6 +58,7 @@ bool          recins_force     = false;
 bool          recins_verbose   = false;
 bool          recins_external  = true;
 bool          recins_auto      = true;
+char         *recins_password  = NULL;
 
 /*
  * Command line options management
@@ -71,7 +74,8 @@ enum
   VERBOSE_ARG,
   NO_EXTERNAL_ARG,
   RECORD_ARG,
-  NO_AUTO_ARG
+  NO_AUTO_ARG,
+  PASSWORD_ARG
 };
 
 static const struct option GNU_longOptions[] =
@@ -86,6 +90,7 @@ static const struct option GNU_longOptions[] =
     {"no-external", no_argument, NULL, NO_EXTERNAL_ARG},
     {"record", required_argument, NULL, RECORD_ARG},
     {"no-auto", no_argument, NULL, NO_AUTO_ARG},
+    {"password", required_argument, NULL, PASSWORD_ARG},
     {NULL, 0, NULL, 0}
   };
 
@@ -120,6 +125,14 @@ Insert new records in a rec database.\n"), stdout);
       --verbose                       give a detailed report if the integrity check\n\
                                         fails.\n"), stdout);
 
+#if defined REC_CRYPT_SUPPORT
+  /* TRANSLATORS: --help output, encryption related options.
+     no-wrap */
+  fputs (_("\
+  -s, --password=STR                  encrypt confidential fields with the given password.\n"),
+         stdout);
+#endif
+
   recutl_print_help_common ();
 
   puts ("");
@@ -212,7 +225,12 @@ recins_add_auto_field_date (rec_rset_t rset,
 
   t = time (NULL);
   tmp = localtime (&t);
+
+  setlocale (LC_TIME, "C"); /* We want english dates that can be
+                                 parsed with parse_datetime */
   strftime (outstr, sizeof(outstr), "%a, %d %b %Y %T %z", tmp);
+  setlocale (LC_TIME, ""); /* And restore the locale from the
+                              environment. */
 
   auto_field = rec_field_new (rec_field_name_dup (field_name),
                               outstr);
@@ -241,9 +259,7 @@ recins_add_auto_fields (rec_rset_t rset,
     {
       if ((auto_fields = rec_rset_auto (rset)))
         {
-          for (i = (rec_fex_size (auto_fields) - 1);
-               i >= 0;
-               i--)
+          for (i = 0; i < rec_fex_size (auto_fields); i++)
             {
               auto_field_name = rec_fex_elem_field_name (rec_fex_get (auto_fields, i));
               
@@ -312,6 +328,29 @@ recins_insert_record (rec_db_t db,
     {
       /* Add auto-set fields required by this record set.  */
       record_to_insert = recins_add_auto_fields (rset, record);
+      rec_record_destroy (record);
+
+#if defined REC_CRYPT_SUPPORT
+      {
+        /* Encrypt the value of fields declared as confidential in
+           this record set.  */
+        
+        rec_fex_t confidential_fields =
+          rec_rset_confidential (rset);
+        if (!recins_password && (rec_fex_size (confidential_fields) > 0))
+          {
+            recutl_warning (_("the record set contains confidential fields but no password was provided\n"));
+            recutl_warning (_("the resulting record will have those fields unencrypted!\n"));
+          }
+        else
+          {
+            if (!rec_encrypt_record (rset, record_to_insert, recins_password))
+              {
+                recutl_fatal ("encrypting a record.  Please report this.\n");
+              }
+          }
+      }
+#endif
 
       new_elem = rec_rset_elem_record_new (rset, record_to_insert);
 
@@ -373,7 +412,7 @@ void recins_parse_args (int argc,
   while ((ret = getopt_long (argc,
                              argv,
                              RECORD_SELECTION_SHORT_ARGS
-                             "f:v:r:",
+                             "f:v:r:s:",
                              GNU_longOptions,
                              NULL)) != -1)
     {
@@ -454,6 +493,17 @@ void recins_parse_args (int argc,
             recins_auto = false;
             break;
           }
+        case PASSWORD_ARG:
+        case 's':
+          {
+            if (recins_password != NULL)
+              {
+                recutl_fatal (_("more than one password was specified\n"));
+              }
+
+            recins_password = xstrdup (optarg);
+            break;
+          }
         case RECORD_ARG:
         case 'r':
           {
@@ -462,7 +512,6 @@ void recins_parse_args (int argc,
             if (!provided_record)
               {
                 recutl_fatal (_("error while parsing the record provided by -r\n"));
-                exit (EXIT_FAILURE);
               }
 
             if (recins_record)
@@ -536,6 +585,29 @@ recins_add_new_record (rec_db_t db)
      
           /* Add auto-set fields required by this record set.  */
           record_to_insert = recins_add_auto_fields (rset, recins_record);
+          rec_record_destroy (recins_record);
+
+#if defined REC_CRYPT_SUPPORT
+      {
+        /* Encrypt the value of fields declared as confidential in
+           this record set.  */
+        
+        rec_fex_t confidential_fields =
+          rec_rset_confidential (rset);
+        if (!recins_password && (rec_fex_size (confidential_fields) > 0))
+          {
+            recutl_warning (_("the record set contains confidential fields but no password was provided\n"));
+            recutl_warning (_("the resulting record will have those fields unencrypted!\n"));
+          }
+        else
+          {
+            if (!rec_encrypt_record (rset, record_to_insert, recins_password))
+              {
+                recutl_fatal ("encrypting a record.  Please report this.\n");
+              }
+          }
+      }
+#endif
 
           rset_elem = rec_rset_first_record (rset);
           while (rec_rset_elem_p (rset_elem))
@@ -588,6 +660,11 @@ main (int argc, char *argv[])
   recins_parse_args (argc, argv);
 
   db = recutl_read_db_from_file (recins_file);
+  if (!db)
+    {
+      /* Create an empty database.  */
+      db = rec_db_new ();
+    }
   recins_add_new_record (db);
   recutl_write_db_to_file (db, recins_file);
 
