@@ -1,4 +1,4 @@
-/* -*- mode: C -*- Time-stamp: "2011-01-31 22:35:08 jemarch"
+/* -*- mode: C -*-
  *
  *       File:         recutl.c
  *       Date:         Thu Apr 22 17:30:48 2010
@@ -35,11 +35,18 @@
 #include <locale.h>
 #include <gettext.h>
 #define _(str) gettext (str)
+#if defined REC_CRYPT_SUPPORT
+#   include <gcrypt.h>
+#endif
 #include <progname.h>
 
 #include <rec.h>
 #include <recutl.h>
 
+bool             recutl_sort_p         = false;
+char             *recutl_order_rset    = NULL;
+rec_field_name_t recutl_order_by_field = NULL;
+
 void recutl_print_help (void); /* Forward prototype.  */
 
 void
@@ -47,6 +54,12 @@ recutl_init (char *util_name)
 {
   set_program_name (xstrdup (util_name));
 
+#if defined REC_CRYPT_SUPPORT
+  /* Initialize libgcrypt */
+  gcry_check_version (NULL);
+  gcry_control (GCRYCTL_INITIALIZATION_FINISHED, 0);
+#endif
+
   /* Initialize librec */
   rec_init ();
 
@@ -184,8 +197,10 @@ recutl_parse_db_from_file (FILE *in,
   rec_parser_t parser;
 
   res = true;
-  parser = rec_parser_new (in, file_name);
 
+  parser = rec_parser_new (in, file_name);
+  rec_parser_set_ordered (parser, recutl_sort_p);
+  rec_parser_sort_rset (parser, recutl_order_rset, recutl_order_by_field);
   while (rec_parse_rset (parser, &rset))
     {
       char *rset_type;
@@ -296,8 +311,7 @@ recutl_read_db_from_file (char *file_name)
       in = fopen (file_name, "r");
       if (in == NULL)
         {
-          recutl_fatal (_("cannot read file %s\n"), file_name);
-          exit (EXIT_FAILURE);
+          return NULL;
         }
     }
   else
@@ -429,4 +443,13 @@ recutl_check_integrity (rec_db_t db,
     }
 }
 
+void
+recutl_sorting_parser (bool sort_p,
+                       char *rset_name,
+                       rec_field_name_t field_name)
+{
+  recutl_sort_p = sort_p;
+  recutl_order_by_field = field_name;
+}
+
 /* End of recutl.c */
